Package: movecost
Title: Calculation of Slope-Dependant Accumulated Cost Surface,
        Least-Cost Paths, Least-Cost Corridors, Least-Cost Networks
        Related to Human Movement Across the Landscape
Version: 2.2
Authors@R: person("Gianmarco", "Alberti", email = "gianmarcoalberti@gmail.com", role = c("aut", "cre"))
Description: Provides the facility to calculate non-isotropic accumulated cost surface, least-cost paths, least-cost corridors, least-cost networks using a number of human-movement-related cost functions that can be selected by the user. It just requires a Digital Terrain Model, a start location and (optionally) destination locations. See Alberti (2019) <doi:10.1016/j.softx.2019.100331>.
Depends: R (>= 4.0.0)
Imports: chron (>= 2.3-56), gdistance (>= 1.2-2), elevatr (>= 0.3.4),
        Matrix (>= 1.5.0), methods (>= 4.0.3), raster (>= 2.8-4), sf
        (>= 1.0-9), sp (>= 1.4.0), terra (>= 1.3.0), utils (>= 4.0.0)
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.2.3
NeedsCompilation: no
Packaged: 2026-04-29 17:01:27 UTC; gianmarcoalberti
Author: Gianmarco Alberti [aut, cre]
Maintainer: Gianmarco Alberti <gianmarcoalberti@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-29 17:20:02 UTC
Built: R 4.6.0; ; 2026-04-29 20:53:26 UTC; unix
