# Generated by roxygen2: do not edit by hand

export(movealloc)
export(movebound)
export(movecomp)
export(movecorr)
export(movecost)
export(movenetw)
export(moverank)
importFrom(Matrix,Matrix)
importFrom(chron,times)
importFrom(elevatr,get_elev_raster)
importFrom(grDevices,grey)
importFrom(grDevices,terrain.colors)
importFrom(grDevices,topo.colors)
importFrom(graphics,boxplot)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,symbols)
importFrom(graphics,text)
importFrom(methods,as)
importFrom(methods,new)
importFrom(raster,cellStats)
importFrom(raster,crop)
importFrom(raster,crs)
importFrom(raster,extend)
importFrom(raster,extent)
importFrom(raster,focal)
importFrom(raster,mask)
importFrom(raster,ncell)
importFrom(raster,projection)
importFrom(raster,raster)
importFrom(raster,rasterize)
importFrom(raster,stack)
importFrom(raster,terrain)
importFrom(raster,which.min)
importFrom(sf,st_area)
importFrom(sf,st_as_sf)
importFrom(sf,st_length)
importFrom(sp,SpatialLinesDataFrame)
importFrom(stats,na.omit)
importFrom(terra,as.polygons)
importFrom(terra,rast)
importFrom(terra,rasterizeGeom)
importFrom(terra,vect)
importFrom(terra,writeVector)
importFrom(utils,combn)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
