#' Dataset: Malta DTM (40m cell size)
#'
#' A RasterLayer representing a Digital Terrain Model of Malta (40m resolution).\cr
#'
#'
#' @docType data
#' @keywords datasets
#' @name malta_dtm_40
#' @usage data(malta_dtm_40)
#' @format RasterLayer
NULL
