#' Dataset: raster dataset representing the elevation of the volcano Maunga Whau (Auckland, New Zealand)
#'
#' A RasterLayer representing a DTM of the volcano Maunga Whau (Auckland, New Zealand).
#'
#'
#' @docType data
#' @keywords datasets
#' @name volc
#' @usage data(volc)
#' @format RasterLayer
NULL
