#' Dataset: location on Mount Etna (Sicily, Italy)
#'
#' A SpatialPointsDataFrame representing a spot on Mount Etna, to be used as start location for least-cost paths calculation.
#'
#'
#' @docType data
#' @keywords datasets
#' @name Etna_start_location
#' @usage data(Etna_start_location)
#' @format SpatialPointsDataFrame
NULL
