#' Dataset: locations on the volcano Maunga Whau (Auckland, New Zealand)
#'
#' A SpatialPointsDataFrame representing spots on the volcano Maunga Whau (Auckland, New Zealand), to be used as destination locations for least-cost paths calculation.
#'
#'
#' @docType data
#' @keywords datasets
#' @name destin.loc
#' @usage data(destin.loc)
#' @format SpatialPointsDataFrame
NULL
