#' Dataset: location on the volcano Maunga Whau (Auckland, New Zealand)
#'
#' A SpatialPointsDataFrame representing a spot on the volcano Maunga Whau (Auckland, New Zealand).
#'
#'
#' @docType data
#' @keywords datasets
#' @name volc.loc
#' @usage data(volc.loc)
#' @format SpatialPointsDataFrame
NULL
