#' Dataset: bounding polygon representing a study area on Mount Etna (Sicily, Italy)
#'
#' A SpatialPolygonDataFrame representing an area on Mount Etna (Sicily, Italy), to be used to download elevation data.
#'
#'
#' @docType data
#' @keywords datasets
#' @name Etna_boundary
#' @usage data(Etna_boundary)
#' @format SpatialPolygonDataFrame
NULL
