#' Dataset: locations on Mount Etna (Sicily, Italy)
#'
#' A SpatialPointsDataFrame representing spots on Mount Etna (Sicily, Italy), to be used as destination locations for least-cost paths calculation.
#'
#'
#' @docType data
#' @keywords datasets
#' @name Etna_end_location
#' @usage data(Etna_end_location)
#' @format SpatialPointsDataFrame
NULL
