#' Dataset: location of springs in Malta
#'
#' A SpatialPointsDataFrame representing the location of spring-related toponyms in Malta.
#'
#'
#' @docType data
#' @keywords datasets
#' @name springs
#' @usage data(springs)
#' @format SpatialPointsDataFrame
NULL
