% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/destin.loc.r
\docType{data}
\name{destin.loc}
\alias{destin.loc}
\title{Dataset: locations on the volcano Maunga Whau (Auckland, New Zealand)}
\format{
SpatialPointsDataFrame
}
\usage{
data(destin.loc)
}
\description{
A SpatialPointsDataFrame representing spots on the volcano Maunga Whau (Auckland, New Zealand), to be used as destination locations for least-cost paths calculation.
}
\keyword{datasets}
